#############################################################################
# Author:                                                                   #
# ------                                                                    #
#  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  #
#  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    #
#  Jozef Stefan Institute                          Fax: x 386 1 477 3811    #
#  Jamova 39, SI-1000 Ljubljana                                             #
#  SLOVENIA                                                                 #
#                                                                           #
# Source: $XCRYSDEN_TOPDIR/Tcl/kPath.tcl                                       
# ------                                                                    #
# Copyright (c) 1996-2003 by Anton Kokalj                                   #
#############################################################################

proc kPath {} {
    global kpath prop BzOK system
   
    set BzOK(done) 0
    if { ! [Bz_MakeToplevel generic] } {
	return
    }

    #if !$BzOK(done) {
    #	# cancel button was pressed
    #	set kpath(active) 0
    #	return
    #}

    # so OK button was pressed
    # save prop(c95_BAND_script)
    set filetypes {
	{{k-Path File                }   {.kpf}  }
	{{CRYSTAL Property Input File}   {.d3 }  }
	{{All Files                  }   {*   }  }
    } 

    set sfile [tk_getSaveFile -initialdir $system(PWD) \
		   -title "Save k-Path" \
		   -defaultextension .kpf \
		   -filetypes $filetypes]

    # maybe Cancel button was pressed
    if { $sfile == {} } { 
	set kpath(active) 0
	return 
    }

    #
    # Just kPath & CRYSTAL d3 file should be supported here !!!
    #
    if { [file extension $sfile] == {.kpf} } {
	#
	# XCRYSDEN's kPath file 
	#

	set content {
#------------------------------------------------------------------------
# This is an XCRYSDEN k-Path File. Here you will find the
# data about the selected special k-point coordinates !!!
#------------------------------------------------------------------------

#------------------------------------------------------------------------
# WARNING: 
#   special k-point coordinates in this file are in "CRYSTAL" 
#   (i.e. fractional) coordinates, which are expressed either 
#   in the basis of "primitive" or "conventional" reciprocal 
#   lattice vectors. See below !!!
#------------------------------------------------------------------------


k-point coordinates are expressed in the basis: 
}
    
	append content [format "%s\n" $kpath(basis)]
	
	append content {
#
# #-------------------------------------#
# # INTEGER FORM of k-point COORDINATES #
# #-------------------------------------#
#
# some programs want an integer form of k-point coordinates,
# this looks like:
#
# k-point == (M*kx, M*ky, M*kz), where M is an integer multiplier
#
# Actual k-point coordinates are then calculated as:
#
# k-point == 1/M * (M*kx, M*ky, M*kz)
#------------------------------------------------------------------------

Multiplier M: }
	append content [format "%s\n" $BzOK(iss)]
	
	append content "\nInteger form of k-point coordinates (M*kx,M*ky,M*kz,label):\n"
	append content [format "%s\n" $kpath(point_labels)]
	
	append content {
#
# #----------------------------------#
# # REAL FORM of k-point COORDINATES #
# #----------------------------------#
#
	}

	append content "\nReal form of k-point coordinates (kx,ky,kz,label):\n"
	foreach line [split $kpath(point_labels) \n] {
	    if { [llength $line] >= 3 } {
		for {set i 0} {$i < 3} {incr i} {
		    set k($i) \
			[expr double([lindex $line $i]) / double($BzOK(iss))]
		}
		set label [lindex $line 3]
		if { $label == {} } {
		    set label A
		}
		append content [format "%15.10f  %15.10f  %15.10f     %s\n" \
				    $k(0) $k(1) $k(2) $label]
	    }
	}
	append content {
#
# #-------------#
# # END of FILE #
# #-------------#
#
	}
    } else {
	#
	# write CRYSTAL d3 file
	#
	set content $prop(c95_BAND_script)
    }
    
    WriteFile $sfile $content w
    set kpath(active) 0
}
